// ----------------------------------------------------------------------------
// Logical.java
// Logical constants true and false
// $Id: Logical.java,v 1.10 2006/02/23 19:51:04 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2005, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
// ----------------------------------------------------------------------------
package fmrisc.Syntax;

import java.io.PrintWriter;

/**
 * 
 * Logical constants.
 *
 */
public final class Logical extends ExpressionBase
{
  private boolean value;
  
  /***************************************************************************
   * construct logical constant with denoted value
   * @param value the logical value
   **************************************************************************/
  public Logical(boolean value)
  {
    this.value = value;
  }
  
  /***************************************************************************
   * get binding level for printing (lower numbers mean greater binding power)
   * @return the priority level
   **************************************************************************/
  public int getPriority()
  {
    return 0;
  }
  
  /****************************************************************************
   * get value of constant
   ***************************************************************************/
  public boolean getValue()
  {
    return value;
  }

  /***************************************************************************
   * Prints text representation of tree on out (without new line termination).
   * @param out the stream on which the text is written
   **************************************************************************/
  public void printCore(PrintWriter out)
  {
    if (value)
      out.print("TRUE");
    else
      out.print("FALSE");
  }
  
  /***************************************************************************
   * Prints text representation on out (without new line termination).
   * Overrides ASTBase.print such that parentheses are never printed
   * around the constant.
   * @param out the stream on which the text is written
   **************************************************************************/
  public void print(PrintWriter out)
  {
    printCore(out);
  }
  
  /***************************************************************************
   * return copy of AST with value references instantiated by the substitutions
   * set in the corresponding declaration identifiers
   * @return the instantiated copy
   **************************************************************************/
  public Expression instantiate()
  {
    return this;
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   * @return the result of the visit (may be null)
   **************************************************************************/
  public AST accept(ASTVisitor visitor) 
  { 
    AST result = visitor.visit(this);
    if (result != null) return result;
    if (!visitor.isCloned()) return null;
    return this;
  }
}
